import os

from PySide2.QtCore import QFile
from PySide2.QtGui import QDoubleValidator
from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QWidget

from axipy import view_manager, CoordSystem, Pnt, Point

from RotateMap.ui.tools.rotetMapTool import Rotate, calcAffineMapInfo


class FormRotateMap(QWidget):
    def __init__(self,parent=None):
        self.__parentWin=parent
        self.load_ui("RotateMap.ui")
        self.__tool_rotate=Rotate()
        self.window.ln_x.setValidator(QDoubleValidator())
        self.window.ln_y.setValidator(QDoubleValidator())
        self.window.ch_center_map.stateChanged.connect(self.__change_center_map)
        self.window.db_spin_rotate.valueChanged.connect(self.__change_angle)
        self.window.pb_rotate.clicked.connect(self.__run_rotate)
        self.window.pb_cancel_rotate.clicked.connect(self.__undo_run_rotate)
    def load_ui(self,name_resource):
        loader = QUiLoader()
        path = os.path.join(os.path.dirname(__file__),name_resource)
        ui_file = QFile(path)
        ui_file.open(QFile.ReadOnly)
        self.window = loader.load(ui_file,self.__parentWin)
        ui_file.close()
    def __change_center_map(self):
        status=self.window.ch_center_map.isChecked()
        if not status:
            self.window.ln_x.setReadOnly(False)
            self.window.ln_y.setReadOnly(False)
        else:
            self.window.ln_x.setReadOnly(True)
            self.window.ln_y.setReadOnly(True)

            self.updateActiveMap(view_manager.active)

        jkl=0
    def __change_angle(self,value):
        str_value=str(value)
        if str_value=="0.0" or str_value=="0,0":
            self.window.pb_rotate.setEnabled(False)
        else:
            self.window.pb_rotate.setEnabled(True)
    def updateActiveMap(self,active_map):
        self.__curent_map_view=active_map
        if self.window.ch_center_map.isChecked():
            center_map_4326=self.__tool_rotate.center(self.__curent_map_view)
            str_x=None
            try:
                str_x='{:.6f}'.format(center_map_4326.x)
                self.window.ln_x.setText(str_x)
                self.window.ln_x.setCursorPosition(0)
            except:
                pass
            try:
                str_x='{:.6f}'.format(center_map_4326.y)
                self.window.ln_y.setText(str_x)
                self.window.ln_y.setCursorPosition(0)
            except:
                pass
    def __run_rotate(self):
        str_x_center=self.window.ln_x.text()
        str_y_center=self.window.ln_y.text()
        center_org=self.__curent_map_view.center
        try:
            x_center=float(str_x_center)
            y_center=float(str_y_center)

        except:
            pass
        if self.window.ch_center_map.isChecked():
            center_map_4326=self.__tool_rotate.center(self.__curent_map_view)
            x_center=center_map_4326.x
            y_center=center_map_4326.y
        angle=self.window.db_spin_rotate.value()
        '''
        if angle<0:
            angle=360.0+angle
        '''


        parametrs_rotate=calcAffineMapInfo(x_center,y_center,angle)
        if parametrs_rotate is None:
            return
        str_cs_prj=self.__curent_map_view.coordsystem.prj
        cs_original=self.__curent_map_view.coordsystem
        str_unit_cs=self.__curent_map_view.coordsystem.unit.name
        cs_bounds=str_cs_prj.split("Bounds")
        str_bounds=None
        if len(cs_bounds)>1:
            str_bounds="Bounds "+cs_bounds[1]
        patern_affine='Affine Units "{unit}" {aa:.6f},{ab:.6f},{ac:.8f},{ad:.6f},{ae:.6f},{af:.8f}'
        str_affine=patern_affine.format(unit=str_unit_cs,aa=parametrs_rotate[0],ab=parametrs_rotate[1],ac=parametrs_rotate[2],ad=parametrs_rotate[3],ae=parametrs_rotate[4],af=parametrs_rotate[5])
        str_new_prj=cs_bounds[0]+" "+str_affine
        if str_bounds is not None:
            str_new_prj=str_new_prj+str_bounds
        new_cs=CoordSystem.from_prj(str_new_prj)
        self.__curent_map_view.coordsystem=new_cs
        if self.window.ch_setafter_rotate.isChecked():
            pt_center=Point(center_org.x,center_org.y,cs_original)
            new_pt_center=pt_center.reproject(new_cs)
            self.__curent_map_view.scale_with_center(1,Pnt(new_pt_center.x,new_pt_center.y))
        self.window.pb_rotate.setEnabled(False)
        self.window.pb_cancel_rotate.setEnabled(True)
    def __undo_run_rotate(self):
        str_cs_prj=self.__curent_map_view.coordsystem.prj
        str_unit_cs=self.__curent_map_view.coordsystem.unit.name
        cs_bounds=str_cs_prj.split("Bounds")
        str_bounds=None
        if len(cs_bounds)>1:
            str_bounds="Bounds "+cs_bounds[1]
        cs_affine=str_cs_prj.split("Affine")
        if len(cs_affine)<=1:
            ''' параметров афинного перобразования нет '''
            return
        str_new_cs=cs_affine[0]
        if str_bounds is not None:
            str_new_cs=str_new_cs+" "+str_bounds
        new_cs=CoordSystem.from_prj(str_new_cs)
        self.__curent_map_view.coordsystem=new_cs
        self.window.pb_rotate.setEnabled(True)
        self.window.pb_cancel_rotate.setEnabled(False)



    @property
    def widget(self):
        return self.window
        #return self
    def show(self):
        #self.ui.showMaximized()
        #print("Show Form")
        self.window.show()
